@@ -21,6 +21,7 @@
 #include <libxml/xmlIO.h>
 #include <libxml/SAX.h>
 #include <libxml/uri.h>
+#include <libxml/valid.h>
 #include <libxml/HTMLtree.h>
 
 /* #define DEBUG_SAX */
@@ -611,6 +612,7 @@ unparsedEntityDecl(void *ctx, const xmlChar *name,
 		   const xmlChar *publicId, const xmlChar *systemId,
 		   const xmlChar *notationName)
 {
+    xmlEntityPtr ent;
     xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
 #ifdef DEBUG_SAX
     xmlGenericError(xmlGenericErrorContext,
@@ -621,15 +623,47 @@ unparsedEntityDecl(void *ctx, const xmlChar *name,
         ctxt->myDoc && ctxt->myDoc->extSubset)
 	ctxt->valid &= xmlValidateNotationUse(&ctxt->vctxt, ctxt->myDoc,
 	                                      notationName);
-    if (ctxt->inSubset == 1)
-	xmlAddDocEntity(ctxt->myDoc, name,
+    if (ctxt->inSubset == 1) {
+	ent = xmlAddDocEntity(ctxt->myDoc, name,
 			XML_EXTERNAL_GENERAL_UNPARSED_ENTITY,
 			publicId, systemId, notationName);
-    else if (ctxt->inSubset == 2)
-	xmlAddDtdEntity(ctxt->myDoc, name,
+	if ((ent == NULL) && (ctxt->pedantic) &&
+	    (ctxt->sax != NULL) && (ctxt->sax->warning != NULL))
+	    ctxt->sax->warning(ctxt, 
+	     "Entity(%s) already defined in the internal subset\n", name);
+	if ((ent != NULL) && (ent->URI == NULL) && (systemId != NULL)) {
+	    xmlChar *URI;
+	    const char *base = NULL;
+
+	    if (ctxt->input != NULL)
+		base = ctxt->input->filename;
+	    if (base == NULL)
+		base = ctxt->directory;
+	
+	    URI = xmlBuildURI(systemId, (const xmlChar *) base);
+	    ent->URI = URI;
+	}
+    } else if (ctxt->inSubset == 2) {
+	ent = xmlAddDtdEntity(ctxt->myDoc, name,
 			XML_EXTERNAL_GENERAL_UNPARSED_ENTITY,
 			publicId, systemId, notationName);
-    else {
+	if ((ent == NULL) && (ctxt->pedantic) &&
+	    (ctxt->sax != NULL) && (ctxt->sax->warning != NULL))
+	    ctxt->sax->warning(ctxt, 
+	     "Entity(%s) already defined in the external subset\n", name);
+	if ((ent != NULL) && (ent->URI == NULL) && (systemId != NULL)) {
+	    xmlChar *URI;
+	    const char *base = NULL;
+
+	    if (ctxt->input != NULL)
+		base = ctxt->input->filename;
+	    if (base == NULL)
+		base = ctxt->directory;
+	
+	    URI = xmlBuildURI(systemId, (const xmlChar *) base);
+	    ent->URI = URI;
+	}
+    } else {
 	if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
 	    ctxt->sax->error(ctxt, 
 	     "SAX.unparsedEntityDecl(%s) called while not in subset\n", name);
@@ -999,6 +1033,78 @@ startElement(void *ctx, const xmlChar *fullname, const xmlChar **atts)
 	}
     }
 
+    /*
+     * Insert all the defaulted attributes from the DTD especially namespaces
+     */
+    if ((!ctxt->html) &&
+	((ctxt->myDoc->intSubset != NULL) ||
+	 (ctxt->myDoc->extSubset != NULL))) {
+	xmlElementPtr elemDecl = NULL;
+
+	if (prefix != NULL) {
+	    if (ctxt->myDoc->intSubset != NULL)
+		elemDecl = xmlGetDtdQElementDesc(ctxt->myDoc->intSubset,
+			                         name, prefix);
+	    if ((elemDecl == NULL) && (ctxt->myDoc->extSubset != NULL))
+		elemDecl = xmlGetDtdQElementDesc(ctxt->myDoc->extSubset,
+			                         name, prefix);
+	} else {
+	    if (ctxt->myDoc->intSubset != NULL)
+		elemDecl = xmlGetDtdQElementDesc(ctxt->myDoc->intSubset,
+			                         name, prefix);
+	    if ((elemDecl == NULL) && (ctxt->myDoc->extSubset != NULL))
+		elemDecl = xmlGetDtdQElementDesc(ctxt->myDoc->extSubset,
+			                         name, prefix);
+	}
+	if (elemDecl != NULL) {
+	    xmlAttributePtr  attr = elemDecl->attributes;
+	    while (attr != NULL) {
+		if (attr->defaultValue != NULL) {
+		    /*
+		     * the element should be instanciated in the tree if:
+		     *  - this is a namespace prefix
+		     *  - the user required for completion in the tree
+		     *    like XSLT
+		     */
+		    if (((attr->prefix != NULL) &&
+			 (xmlStrEqual(attr->prefix, BAD_CAST "xmlns"))) ||
+		        ((attr->prefix == NULL) &&
+			 (xmlStrEqual(attr->name, BAD_CAST "xmlns"))) ||
+			(ctxt->loadsubset & XML_COMPLETE_ATTRS)) {
+			xmlChar buffer[100];
+			const xmlChar *fulln = attr->name;
+
+			if (attr->prefix != NULL) {
+			    snprintf((char *) buffer, 99, "%s:%s",
+				     attr->prefix, attr->name);
+			    buffer[99] = 0;
+			    fulln = buffer;
+			}
+
+			/*
+			 * Check that the attribute is not declared in the
+			 * serialization
+			 */
+			att = NULL;
+			if (atts != NULL) {
+			    i = 0;
+			    att = atts[i];
+			    while (att != NULL) {
+				if (xmlStrEqual(att, fulln))
+				    break;
+				i += 2;
+				att = atts[i];
+			    }
+			}
+			if (att == NULL)
+			    attribute(ctxt, fulln, attr->defaultValue);
+		    }
+		}
+		attr = attr->nexth;
+	    }
+	}
+    }
+
     /*
      * process all the attributes whose name start with "xml"
      */
@@ -1201,8 +1307,10 @@ characters(void *ctx, const xmlChar *ch, int len)
 	}
 #endif
     } else {
-	int isText = xmlNodeIsText(lastChild);
-	if ((isText) && (ctxt->nodemem != 0)) {
+	int coalesceText = (lastChild != NULL) &&
+	    (lastChild->type == XML_TEXT_NODE) &&
+	    (lastChild->name == xmlStringText);
+	if ((coalesceText) && (ctxt->nodemem != 0)) {
 #ifndef XML_USE_BUFFER_CONTENT
 	    /*
 	     * The whole point of maintaining nodelen and nodemem,
@@ -1233,7 +1341,7 @@ characters(void *ctx, const xmlChar *ch, int len)
 #else
 	    xmlTextConcat(lastChild, ch, len);
 #endif
-	} else if (isText) {
+	} else if (coalesceText) {
 	    xmlTextConcat(lastChild, ch, len);
 	    if (ctxt->node->children != NULL) {
 		ctxt->nodelen = xmlStrlen(lastChild->content);
