@@ -185,9 +185,13 @@ LIBXML_DLL_IMPORT extern int xmlDoValidityCheckingDefaultValue;
 LIBXML_DLL_IMPORT extern int xmlLoadExtDtdDefaultValue;
 LIBXML_DLL_IMPORT extern int xmlPedanticParserDefaultValue;
 LIBXML_DLL_IMPORT extern int xmlKeepBlanksDefaultValue;
-LIBXML_DLL_IMPORT extern xmlChar xmlStringText[];
-LIBXML_DLL_IMPORT extern xmlChar xmlStringTextNoenc[];
-LIBXML_DLL_IMPORT extern xmlChar xmlStringComment[];
+
+/**
+ * Global variables used for predefined strings
+ */
+LIBXML_DLL_IMPORT extern const xmlChar xmlStringText[];
+LIBXML_DLL_IMPORT extern const xmlChar xmlStringTextNoenc[];
+LIBXML_DLL_IMPORT extern const xmlChar xmlStringComment[];
 
 /*
  * Function to finish teh work of the macros where needed
@@ -322,12 +326,29 @@ void			xmlParseMisc		(xmlParserCtxtPtr ctxt);
 void			xmlParseExternalSubset	(xmlParserCtxtPtr ctxt,
 						 const xmlChar *ExternalID,
 						 const xmlChar *SystemID); 
-/*
- * Entities substitution
+/**
+ * XML_SUBSTITUTE_NONE:
+ *
+ * If no entities need to be substitued
  */
 #define XML_SUBSTITUTE_NONE	0
+/**
+ * XML_SUBSTITUTE_REF:
+ *
+ * Whether general entities need to be substitued
+ */
 #define XML_SUBSTITUTE_REF	1
+/**
+ * XML_SUBSTITUTE_PEREF:
+ *
+ * Whether parameter entities need to be substitued
+ */
 #define XML_SUBSTITUTE_PEREF	2
+/**
+ * XML_SUBSTITUTE_BOTH:
+ *
+ * Both general and parameter entities need to be substitued
+ */
 #define XML_SUBSTITUTE_BOTH 	3
 
 xmlChar *		xmlDecodeEntities	(xmlParserCtxtPtr ctxt,
