@@ -92,26 +92,19 @@ static int push = 0;
 static int memory = 0;
 #endif
 static int noblanks = 0;
+static int format = 0;
 static int testIO = 0;
 static char *encoding = NULL;
 #ifdef LIBXML_XINCLUDE_ENABLED
 static int xinclude = 0;
 #endif
+static int dtdattrs = 0;
 static int loaddtd = 0;
 static int progresult = 0;
 static int timing = 0;
 static int generate = 0;
 static struct timeval begin, end;
 
-
-#ifdef VMS
-extern int xmlDoValidityCheckingDefaultVal;
-#define xmlDoValidityCheckingDefaultValue xmlDoValidityCheckingDefaultVal
-#else
-extern int xmlDoValidityCheckingDefaultValue;
-#endif
-extern int xmlGetWarningsDefaultValue;
-
 /************************************************************************
  * 									*
  * 			HTML ouput					*
@@ -638,7 +631,10 @@ static void parseAndPrintFile(char *filename) {
 		if (encoding != NULL) {
 		    xmlDocDumpMemoryEnc(doc, &result, &len, encoding);
 		} else {
-		    xmlDocDumpMemory(doc, &result, &len);
+		    if (format)
+			xmlDocDumpFormatMemory(doc, &result, &len, 1);
+		    else
+			xmlDocDumpMemory(doc, &result, &len);
 		}
 		if (result == NULL) {
 		    fprintf(stderr, "Failed to save\n");
@@ -652,6 +648,8 @@ static void parseAndPrintFile(char *filename) {
 		xmlSaveFile("-", doc);
 	    else if (encoding != NULL)
 	        xmlSaveFileEnc("-", doc, encoding);
+	    else if (format)
+		xmlSaveFormatFile("-", doc, 1);
 	    else
 		xmlDocDump(stdout, doc);
 	    if ((timing) && (!repeat)) {
@@ -793,6 +791,7 @@ static void usage(const char *name) {
 #endif
     printf("\t--nowarning : do not emit warnings from parser/validator\n");
     printf("\t--noblanks : drop (ignorable?) blanks spaces\n");
+    printf("\t--format : reformat/reindent the input\n");
     printf("\t--testIO : test user I/O support\n");
     printf("\t--encode encoding : output in the given encoding\n");
 #ifdef LIBXML_CATALOG_ENABLED
@@ -801,8 +800,9 @@ static void usage(const char *name) {
     printf("\t--auto : generate a small doc on the fly\n");
 #ifdef LIBXML_XINCLUDE_ENABLED
     printf("\t--xinclude : do XInclude processing\n");
-    printf("\t--loaddtd : fetch external Dtd\n");
 #endif
+    printf("\t--loaddtd : fetch external Dtd\n");
+    printf("\t--dtdattr : loaddtd + populate the tree with inherited attributes \n");
 }
 int
 main(int argc, char **argv) {
@@ -859,7 +859,11 @@ main(int argc, char **argv) {
 	else if ((!strcmp(argv[i], "-loaddtd")) ||
 	         (!strcmp(argv[i], "--loaddtd")))
 	    loaddtd++;
-	else if ((!strcmp(argv[i], "-valid")) ||
+	else if ((!strcmp(argv[i], "-dtdattr")) ||
+	         (!strcmp(argv[i], "--dtdattr"))) {
+	    loaddtd++;
+	    dtdattrs++;
+	} else if ((!strcmp(argv[i], "-valid")) ||
 	         (!strcmp(argv[i], "--valid")))
 	    valid++;
 	else if ((!strcmp(argv[i], "-postvalid")) ||
@@ -948,6 +952,12 @@ main(int argc, char **argv) {
 	         (!strcmp(argv[i], "--noblanks"))) {
 	     noblanks++;
 	     xmlKeepBlanksDefault(0);
+        }
+	else if ((!strcmp(argv[i], "-format")) ||
+	         (!strcmp(argv[i], "--format"))) {
+	     noblanks++;
+	     format++;
+	     xmlKeepBlanksDefault(0);
 	} else {
 	    fprintf(stderr, "Unknown option %s\n", argv[i]);
 	    usage(argv[0]);
@@ -955,7 +965,10 @@ main(int argc, char **argv) {
 	}
     }
     xmlLineNumbersDefault(1);
-    if (loaddtd != 0) xmlLoadExtDtdDefaultValue = 6; /* fetch DTDs by default */
+    if (loaddtd != 0)
+	xmlLoadExtDtdDefaultValue |= XML_DETECT_IDS;
+    if (dtdattrs)
+	xmlLoadExtDtdDefaultValue |= XML_COMPLETE_ATTRS;
     if (noent != 0) xmlSubstituteEntitiesDefault(1);
     if (valid != 0) xmlDoValidityCheckingDefaultValue = 1;
     if ((htmlout) && (!nowrap)) {
